{% extends "rest_framework/base.html" %}

{% block title %} Django Oscar API {% endblock %}
{% block branding %}
    <span>
        <a href="https://github.com/django-oscar/django-oscar-api/" rel="nofollow" class="navbar-brand">
            Django Oscar API
        </a>
    </span>
{% endblock %}
